function	[out] = rodrigues (in)

% RODRIGUES	Transform rotation matrix into rotation vector and viceversa.
%		
%		Syntax:  [OUT]=RODRIGUES(IN)
% 		If IN is a 3x3 rotation matrix then OUT is the corresponding 3x1 rotation vector
% 		if IN is a rotation 3-vector then OUT is the corresponding 3x3 rotation matrix

%%
%%		Copyright (c) March 1993 -- Pietro Perona
%%		California Institute of Technology
%%

  [m,n] = size (in);

  bigeps = 10e+20*eps;

%% it is a rotation vector

  if ((m==1) & (n==3)) | ((m==3) & (n==1)) 
    theta = norm(in);
    if theta < eps
      R = eye(3);	 
    else
      if n==length(in) in=in'; end; 	%% make it a column vec. if necess.
	 
      omega = in / theta;
	 
      alpha  = cos(theta);
      beta   = sin(theta);
      gamma  = 1-cos(theta);
      omegav = [[0 -omega(3) omega(2)]; [omega(3) 0 -omega(1)]; [-omega(2) omega(1) 0 ]];
      A = omega * omega';
	 
      R = eye(3) * alpha + omegav * beta + A * gamma;	
    end;
    out = R;

%% it is a rotation matrix

  elseif ((m==n) & (m==3) & (norm(in' * in - eye(3)) < bigeps) & (abs(det(in)-1) < bigeps))
    R = in;      
    tr = (trace(R)-1)/2;
    theta = real(acos(tr));
         
    if sin(theta) >= 1e-5,
        
	  vth = 1/(2*sin(theta));
      om1 = [R(3,2)-R(2,3), R(1,3)-R(3,1), R(2,1)-R(1,2)]';
      om  = vth * om1;	 
      out = om * theta;
	 	 
    else
      if tr > 0; % case norm (om) = 0;
        out = [0 0 0]';
      else
        out = theta * (sqrt((diag(R)+1)/2).*[1;2*(R(1,2:3)>=0)'-1]);
      end; 
    end;

%% invalid parameter

  else
    error('Neither a rotation matrix nor a rotation vector were provided');
end;
